@@ -32,6 +32,9 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/bsdtar.c,v 1.93 2008/11/08 04:43:24 kientzle
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#ifdef HAVE_COPYFILE_H
+#include <copyfile.h>
+#endif
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -47,6 +50,9 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/bsdtar.c,v 1.93 2008/11/08 04:43:24 kientzle
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
+#endif
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -60,11 +66,9 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/bsdtar.c,v 1.93 2008/11/08 04:43:24 kientzle
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_ZLIB_H
-#include <zlib.h>
-#endif
 
 #include "bsdtar.h"
+#include "err.h"
 
 /*
  * Per POSIX.1-1988, tar defaults to reading/writing archives to/from
@@ -76,15 +80,48 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/bsdtar.c,v 1.93 2008/11/08 04:43:24 kientzle
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #define	_PATH_DEFTAPE "\\\\.\\tape0"
 #endif
+#if defined(__APPLE__)
+#undef _PATH_DEFTAPE
+#define	_PATH_DEFTAPE "-"  /* Mac OS has no tape support, default to stdio. */
+#endif
 
 #ifndef _PATH_DEFTAPE
 #define	_PATH_DEFTAPE "/dev/tape"
 #endif
 
-/* External function to parse a date/time string (from getdate.y) */
+#ifdef __MINGW32__
+int _CRT_glob = 0; /* Disable broken CRT globbing. */
+#endif
+
+#if defined(HAVE_SIGACTION) && (defined(SIGINFO) || defined(SIGUSR1))
+static volatile int siginfo_occurred;
+
+static void
+siginfo_handler(int sig)
+{
+	(void)sig; /* UNUSED */
+	siginfo_occurred = 1;
+}
+
+int
+need_report(void)
+{
+	int r = siginfo_occurred;
+	siginfo_occurred = 0;
+	return (r);
+}
+#else
+int
+need_report(void)
+{
+	return (0);
+}
+#endif
+
+/* External function to parse a date/time string */
 time_t get_date(time_t, const char *);
 
-static void		 long_help(struct bsdtar *);
+static void		 long_help(void);
 static void		 only_mode(struct bsdtar *, const char *opt,
 			     const char *valid);
 static void		 set_mode(struct bsdtar *, char opt);
@@ -112,33 +149,55 @@ main(int argc, char **argv)
 	bsdtar = &bsdtar_storage;
 	memset(bsdtar, 0, sizeof(*bsdtar));
 	bsdtar->fd = -1; /* Mark as "unused" */
+	bsdtar->gid = -1;
+	bsdtar->uid = -1;
 	option_o = 0;
-#if defined(_WIN32) && !defined(__CYGWIN__)
-	/* Make sure open() function will be used with a binary mode. */
-	/* on cygwin, we need something similar, but instead link against */
-	/* a special startup object, binmode.o */
-	_set_fmode(_O_BINARY);
+
+#if defined(HAVE_SIGACTION)
+	{ /* Set up signal handling. */
+		struct sigaction sa;
+		sa.sa_handler = siginfo_handler;
+		sigemptyset(&sa.sa_mask);
+		sa.sa_flags = 0;
+#ifdef SIGINFO
+		if (sigaction(SIGINFO, &sa, NULL))
+			lafe_errc(1, errno, "sigaction(SIGINFO) failed");
+#endif
+#ifdef SIGUSR1
+		/* ... and treat SIGUSR1 the same way as SIGINFO. */
+		if (sigaction(SIGUSR1, &sa, NULL))
+			lafe_errc(1, errno, "sigaction(SIGUSR1) failed");
+#endif
+#ifdef SIGPIPE
+		/* Ignore SIGPIPE signals. */
+		sa.sa_handler = SIG_IGN;
+		sigaction(SIGPIPE, &sa, NULL);
+#endif
+	}
 #endif
 
-	/* Need bsdtar->progname before calling bsdtar_warnc. */
+
+	/* Need lafe_progname before calling lafe_warnc. */
 	if (*argv == NULL)
-		bsdtar->progname = "bsdtar";
+		lafe_progname = "bsdtar";
 	else {
 #if defined(_WIN32) && !defined(__CYGWIN__)
-		bsdtar->progname = strrchr(*argv, '\\');
+		lafe_progname = strrchr(*argv, '\\');
 #else
-		bsdtar->progname = strrchr(*argv, '/');
+		lafe_progname = strrchr(*argv, '/');
 #endif
-		if (bsdtar->progname != NULL)
-			bsdtar->progname++;
+		if (lafe_progname != NULL)
+			lafe_progname++;
 		else
-			bsdtar->progname = *argv;
+			lafe_progname = *argv;
 	}
 
 	time(&now);
 
+#if HAVE_SETLOCALE
 	if (setlocale(LC_ALL, "") == NULL)
-		bsdtar_warnc(bsdtar, 0, "Failed to set default locale");
+		lafe_warnc(0, "Failed to set default locale");
+#endif
 #if defined(HAVE_NL_LANGINFO) && defined(HAVE_D_MD_ORDER)
 	bsdtar->day_first = (*nl_langinfo(D_MD_ORDER) == 'd');
 #endif
@@ -152,14 +211,21 @@ main(int argc, char **argv)
 	if (bsdtar->filename == NULL)
 		bsdtar->filename = _PATH_DEFTAPE;
 
+	/* Default block size settings. */
+	bsdtar->bytes_per_block = DEFAULT_BYTES_PER_BLOCK;
+	/* Allow library to default this unless user specifies -b. */
+	bsdtar->bytes_in_last_block = -1;
+
 	/* Default: preserve mod time on extract */
 	bsdtar->extract_flags = ARCHIVE_EXTRACT_TIME;
 
 	/* Default: Perform basic security checks. */
 	bsdtar->extract_flags |= SECURITY;
 
-	/* Defaults for root user: */
-	if (bsdtar_is_privileged(bsdtar)) {
+#ifndef _WIN32
+	/* On POSIX systems, assume --same-owner and -p when run by
+	 * the root user.  This doesn't make any sense on Windows. */
+	if (bsdtar->user_uid == 0) {
 		/* --same-owner */
 		bsdtar->extract_flags |= ARCHIVE_EXTRACT_OWNER;
 		/* -p */
@@ -167,7 +233,19 @@ main(int argc, char **argv)
 		bsdtar->extract_flags |= ARCHIVE_EXTRACT_ACL;
 		bsdtar->extract_flags |= ARCHIVE_EXTRACT_XATTR;
 		bsdtar->extract_flags |= ARCHIVE_EXTRACT_FFLAGS;
+		bsdtar->extract_flags |= ARCHIVE_EXTRACT_MAC_METADATA;
 	}
+#endif
+
+	/*
+	 * Enable Mac OS "copyfile()" extension by default.
+	 * This has no effect on other platforms.
+	 */
+	bsdtar->enable_copyfile = 1;
+#ifdef COPYFILE_DISABLE_VAR
+	if (getenv(COPYFILE_DISABLE_VAR))
+		bsdtar->enable_copyfile = 0;
+#endif
 
 	bsdtar->argv = argv;
 	bsdtar->argc = argc;
@@ -184,14 +262,20 @@ main(int argc, char **argv)
 			/* libarchive doesn't need this; just ignore it. */
 			break;
 		case 'b': /* SUSv2 */
-			t = atoi(bsdtar->optarg);
-			if (t <= 0 || t > 1024)
-				bsdtar_errc(bsdtar, 1, 0,
-				    "Argument to -b is out of range (1..1024)");
+			t = atoi(bsdtar->argument);
+			if (t <= 0 || t > 8192)
+				lafe_errc(1, 0,
+				    "Argument to -b is out of range (1..8192)");
 			bsdtar->bytes_per_block = 512 * t;
+			/* Explicit -b forces last block size. */
+			bsdtar->bytes_in_last_block = bsdtar->bytes_per_block;
 			break;
 		case 'C': /* GNU tar */
-			set_chdir(bsdtar, bsdtar->optarg);
+			if (strlen(bsdtar->argument) == 0)
+				lafe_errc(1, 0,
+				    "Meaningless option: -C ''");
+
+			set_chdir(bsdtar, bsdtar->argument);
 			break;
 		case 'c': /* SUSv2 */
 			set_mode(bsdtar, opt);
@@ -202,21 +286,29 @@ main(int argc, char **argv)
 		case OPTION_CHROOT: /* NetBSD */
 			bsdtar->option_chroot = 1;
 			break;
+		case OPTION_DISABLE_COPYFILE: /* Mac OS X */
+			bsdtar->enable_copyfile = 0;
+			break;
 		case OPTION_EXCLUDE: /* GNU tar */
-			if (exclude(bsdtar, bsdtar->optarg))
-				bsdtar_errc(bsdtar, 1, 0,
-				    "Couldn't exclude %s\n", bsdtar->optarg);
+			if (lafe_exclude(&bsdtar->matching, bsdtar->argument))
+				lafe_errc(1, 0,
+				    "Couldn't exclude %s\n", bsdtar->argument);
 			break;
 		case OPTION_FORMAT: /* GNU tar, others */
-			bsdtar->create_format = bsdtar->optarg;
-			break;
-		case OPTION_OPTIONS:
-			bsdtar->option_options = bsdtar->optarg;
+			bsdtar->create_format = bsdtar->argument;
 			break;
 		case 'f': /* SUSv2 */
-			bsdtar->filename = bsdtar->optarg;
-			if (strcmp(bsdtar->filename, "-") == 0)
-				bsdtar->filename = NULL;
+			bsdtar->filename = bsdtar->argument;
+			break;
+		case OPTION_GID: /* cpio */
+			t = atoi(bsdtar->argument);
+			if (t < 0)
+				lafe_errc(1, 0,
+				    "Argument to --gid must be positive");
+			bsdtar->gid = t;
+			break;
+		case OPTION_GNAME: /* cpio */
+			bsdtar->gname = bsdtar->argument;
 			break;
 		case 'H': /* BSD convention */
 			bsdtar->symlink_mode = 'H';
@@ -227,7 +319,7 @@ main(int argc, char **argv)
 			possible_help_request = 1;
 			break;
 		case OPTION_HELP: /* GNU tar, others */
-			long_help(bsdtar);
+			long_help();
 			exit(0);
 			break;
 		case 'I': /* GNU tar */
@@ -241,7 +333,7 @@ main(int argc, char **argv)
 			 * permissions without having to create those
 			 * permissions on disk.
 			 */
-			bsdtar->names_from_file = bsdtar->optarg;
+			bsdtar->names_from_file = bsdtar->argument;
 			break;
 		case OPTION_INCLUDE:
 			/*
@@ -249,36 +341,24 @@ main(int argc, char **argv)
 			 * noone else needs this to filter entries
 			 * when transforming archives.
 			 */
-			if (include(bsdtar, bsdtar->optarg))
-				bsdtar_errc(bsdtar, 1, 0,
+			if (lafe_include(&bsdtar->matching, bsdtar->argument))
+				lafe_errc(1, 0,
 				    "Failed to add %s to inclusion list",
-				    bsdtar->optarg);
+				    bsdtar->argument);
 			break;
 		case 'j': /* GNU tar */
-#if HAVE_LIBBZ2
 			if (bsdtar->create_compression != '\0')
-				bsdtar_errc(bsdtar, 1, 0,
+				lafe_errc(1, 0,
 				    "Can't specify both -%c and -%c", opt,
 				    bsdtar->create_compression);
 			bsdtar->create_compression = opt;
-#else
-			bsdtar_warnc(bsdtar, 0,
-			    "bzip2 compression not supported by this version of bsdtar");
-			usage(bsdtar);
-#endif
 			break;
 		case 'J': /* GNU tar 1.21 and later */
-#if HAVE_LIBLZMA
 			if (bsdtar->create_compression != '\0')
-				bsdtar_errc(bsdtar, 1, 0,
+				lafe_errc(1, 0,
 				    "Can't specify both -%c and -%c", opt,
 				    bsdtar->create_compression);
 			bsdtar->create_compression = opt;
-#else
-			bsdtar_warnc(bsdtar, 0,
-			    "xz compression not supported by this version of bsdtar");
-			usage(bsdtar);
-#endif
 			break;
 		case 'k': /* GNU tar */
 			bsdtar->extract_flags |= ARCHIVE_EXTRACT_NO_OVERWRITE;
@@ -293,18 +373,13 @@ main(int argc, char **argv)
 			/* GNU tar 1.13  used -l for --one-file-system */
 			bsdtar->option_warn_links = 1;
 			break;
-		case OPTION_LZMA:
-#if HAVE_LIBLZMA
+		case OPTION_LZIP: /* GNU tar beginning with 1.23 */
+		case OPTION_LZMA: /* GNU tar beginning with 1.20 */
 			if (bsdtar->create_compression != '\0')
-				bsdtar_errc(bsdtar, 1, 0,
+				lafe_errc(1, 0,
 				    "Can't specify both -%c and -%c", opt,
 				    bsdtar->create_compression);
 			bsdtar->create_compression = opt;
-#else
-			bsdtar_warnc(bsdtar, 0,
-			    "lzma compression not supported by this version of bsdtar");
-			usage(bsdtar);
-#endif
 			break;
 		case 'm': /* SUSv2 */
 			bsdtar->extract_flags &= ~ARCHIVE_EXTRACT_TIME;
@@ -320,28 +395,32 @@ main(int argc, char **argv)
 		 * TODO: Add corresponding "older" options to reverse these.
 		 */
 		case OPTION_NEWER_CTIME: /* GNU tar */
-			bsdtar->newer_ctime_sec = get_date(now, bsdtar->optarg);
+			bsdtar->newer_ctime_filter = 1;
+			bsdtar->newer_ctime_sec = get_date(now, bsdtar->argument);
 			break;
 		case OPTION_NEWER_CTIME_THAN:
 			{
 				struct stat st;
-				if (stat(bsdtar->optarg, &st) != 0)
-					bsdtar_errc(bsdtar, 1, 0,
-					    "Can't open file %s", bsdtar->optarg);
+				if (stat(bsdtar->argument, &st) != 0)
+					lafe_errc(1, 0,
+					    "Can't open file %s", bsdtar->argument);
+				bsdtar->newer_ctime_filter = 1;
 				bsdtar->newer_ctime_sec = st.st_ctime;
 				bsdtar->newer_ctime_nsec =
 				    ARCHIVE_STAT_CTIME_NANOS(&st);
 			}
 			break;
 		case OPTION_NEWER_MTIME: /* GNU tar */
-			bsdtar->newer_mtime_sec = get_date(now, bsdtar->optarg);
+			bsdtar->newer_mtime_filter = 1;
+			bsdtar->newer_mtime_sec = get_date(now, bsdtar->argument);
 			break;
 		case OPTION_NEWER_MTIME_THAN:
 			{
 				struct stat st;
-				if (stat(bsdtar->optarg, &st) != 0)
-					bsdtar_errc(bsdtar, 1, 0,
-					    "Can't open file %s", bsdtar->optarg);
+				if (stat(bsdtar->argument, &st) != 0)
+					lafe_errc(1, 0,
+					    "Can't open file %s", bsdtar->argument);
+				bsdtar->newer_mtime_filter = 1;
 				bsdtar->newer_mtime_sec = st.st_mtime;
 				bsdtar->newer_mtime_nsec =
 				    ARCHIVE_STAT_MTIME_NANOS(&st);
@@ -358,11 +437,14 @@ main(int argc, char **argv)
 			bsdtar->extract_flags &= ~ARCHIVE_EXTRACT_ACL;
 			bsdtar->extract_flags &= ~ARCHIVE_EXTRACT_XATTR;
 			bsdtar->extract_flags &= ~ARCHIVE_EXTRACT_FFLAGS;
+			bsdtar->extract_flags &= ~ARCHIVE_EXTRACT_MAC_METADATA;
 			break;
 		case OPTION_NULL: /* GNU tar */
 			bsdtar->option_null++;
 			break;
 		case OPTION_NUMERIC_OWNER: /* GNU tar */
+			bsdtar->uname = "";
+			bsdtar->gname = "";
 			bsdtar->option_numeric_owner++;
 			break;
 		case 'O': /* GNU tar */
@@ -374,6 +456,9 @@ main(int argc, char **argv)
 		case OPTION_ONE_FILE_SYSTEM: /* GNU tar */
 			bsdtar->option_dont_traverse_mounts = 1;
 			break;
+		case OPTION_OPTIONS:
+			bsdtar->option_options = bsdtar->argument;
+			break;
 #if 0
 		/*
 		 * The common BSD -P option is not necessary, since
@@ -394,6 +479,7 @@ main(int argc, char **argv)
 			bsdtar->extract_flags |= ARCHIVE_EXTRACT_ACL;
 			bsdtar->extract_flags |= ARCHIVE_EXTRACT_XATTR;
 			bsdtar->extract_flags |= ARCHIVE_EXTRACT_FFLAGS;
+			bsdtar->extract_flags |= ARCHIVE_EXTRACT_MAC_METADATA;
 			break;
 		case OPTION_POSIX: /* GNU tar */
 			bsdtar->create_format = "pax";
@@ -409,21 +495,27 @@ main(int argc, char **argv)
 			break;
 		case 's': /* NetBSD pax-as-tar */
 #if HAVE_REGEX_H
-			add_substitution(bsdtar, bsdtar->optarg);
+			add_substitution(bsdtar, bsdtar->argument);
 #else
-			bsdtar_warnc(bsdtar, 0,
+			lafe_warnc(0,
 			    "-s is not supported by this version of bsdtar");
-			usage(bsdtar);
+			usage();
 #endif
 			break;
 		case OPTION_SAME_OWNER: /* GNU tar */
 			bsdtar->extract_flags |= ARCHIVE_EXTRACT_OWNER;
 			break;
 		case OPTION_STRIP_COMPONENTS: /* GNU tar 1.15 */
-			bsdtar->strip_components = atoi(bsdtar->optarg);
+			errno = 0;
+			bsdtar->strip_components = strtol(bsdtar->argument,
+			    NULL, 0);
+			if (errno)
+				lafe_errc(1, 0,
+				    "Invalid --strip-components argument: %s",
+				    bsdtar->argument);
 			break;
 		case 'T': /* GNU tar */
-			bsdtar->names_from_file = bsdtar->optarg;
+			bsdtar->names_from_file = bsdtar->argument;
 			break;
 		case 't': /* SUSv2 */
 			set_mode(bsdtar, opt);
@@ -439,6 +531,16 @@ main(int argc, char **argv)
 		case 'u': /* SUSv2 */
 			set_mode(bsdtar, opt);
 			break;
+		case OPTION_UID: /* cpio */
+			t = atoi(bsdtar->argument);
+			if (t < 0)
+				lafe_errc(1, 0,
+				    "Argument to --uid must be positive");
+			bsdtar->uid = t;
+			break;
+		case OPTION_UNAME: /* cpio */
+			bsdtar->uname = bsdtar->argument;
+			break;
 		case 'v': /* SUSv2 */
 			bsdtar->verbose++;
 			break;
@@ -457,52 +559,40 @@ main(int argc, char **argv)
 			bsdtar->option_interactive = 1;
 			break;
 		case 'X': /* GNU tar */
-			if (exclude_from_file(bsdtar, bsdtar->optarg))
-				bsdtar_errc(bsdtar, 1, 0,
+			if (lafe_exclude_from_file(&bsdtar->matching, bsdtar->argument))
+				lafe_errc(1, 0,
 				    "failed to process exclusions from file %s",
-				    bsdtar->optarg);
+				    bsdtar->argument);
 			break;
 		case 'x': /* SUSv2 */
 			set_mode(bsdtar, opt);
 			break;
 		case 'y': /* FreeBSD version of GNU tar */
-#if HAVE_LIBBZ2
 			if (bsdtar->create_compression != '\0')
-				bsdtar_errc(bsdtar, 1, 0,
+				lafe_errc(1, 0,
 				    "Can't specify both -%c and -%c", opt,
 				    bsdtar->create_compression);
 			bsdtar->create_compression = opt;
-#else
-			bsdtar_warnc(bsdtar, 0,
-			    "bzip2 compression not supported by this version of bsdtar");
-			usage(bsdtar);
-#endif
 			break;
 		case 'Z': /* GNU tar */
 			if (bsdtar->create_compression != '\0')
-				bsdtar_errc(bsdtar, 1, 0,
+				lafe_errc(1, 0,
 				    "Can't specify both -%c and -%c", opt,
 				    bsdtar->create_compression);
 			bsdtar->create_compression = opt;
 			break;
 		case 'z': /* GNU tar, star, many others */
-#if HAVE_LIBZ
 			if (bsdtar->create_compression != '\0')
-				bsdtar_errc(bsdtar, 1, 0,
+				lafe_errc(1, 0,
 				    "Can't specify both -%c and -%c", opt,
 				    bsdtar->create_compression);
 			bsdtar->create_compression = opt;
-#else
-			bsdtar_warnc(bsdtar, 0,
-			    "gzip compression not supported by this version of bsdtar");
-			usage(bsdtar);
-#endif
 			break;
 		case OPTION_USE_COMPRESS_PROGRAM:
-			bsdtar->compress_program = bsdtar->optarg;
+			bsdtar->compress_program = bsdtar->argument;
 			break;
 		default:
-			usage(bsdtar);
+			usage();
 		}
 	}
 
@@ -512,13 +602,13 @@ main(int argc, char **argv)
 
 	/* If no "real" mode was specified, treat -h as --help. */
 	if ((bsdtar->mode == '\0') && possible_help_request) {
-		long_help(bsdtar);
+		long_help();
 		exit(0);
 	}
 
 	/* Otherwise, a mode is required. */
 	if (bsdtar->mode == '\0')
-		bsdtar_errc(bsdtar, 1, 0,
+		lafe_errc(1, 0,
 		    "Must specify one of -c, -r, -t, -u, -x");
 
 	/* Check boolean options only permitted in certain modes. */
@@ -571,8 +661,10 @@ main(int argc, char **argv)
 		buff[1] = bsdtar->symlink_mode;
 		only_mode(bsdtar, buff, "cru");
 	}
-	if (bsdtar->strip_components != 0)
-		only_mode(bsdtar, "--strip-components", "xt");
+
+	/* Filename "-" implies stdio. */
+	if (strcmp(bsdtar->filename, "-") == 0)
+		bsdtar->filename = NULL;
 
 	switch(bsdtar->mode) {
 	case 'c':
@@ -592,13 +684,13 @@ main(int argc, char **argv)
 		break;
 	}
 
-	cleanup_exclusions(bsdtar);
+	lafe_cleanup_exclusions(&bsdtar->matching);
 #if HAVE_REGEX_H
 	cleanup_substitution(bsdtar);
 #endif
 
 	if (bsdtar->return_value != 0)
-		bsdtar_warnc(bsdtar, 0,
+		lafe_warnc(0,
 		    "Error exit delayed from previous errors.");
 	return (bsdtar->return_value);
 }
@@ -607,7 +699,7 @@ static void
 set_mode(struct bsdtar *bsdtar, char opt)
 {
 	if (bsdtar->mode != '\0' && bsdtar->mode != opt)
-		bsdtar_errc(bsdtar, 1, 0,
+		lafe_errc(1, 0,
 		    "Can't specify both -%c and -%c", opt, bsdtar->mode);
 	bsdtar->mode = opt;
 }
@@ -619,18 +711,18 @@ static void
 only_mode(struct bsdtar *bsdtar, const char *opt, const char *valid_modes)
 {
 	if (strchr(valid_modes, bsdtar->mode) == NULL)
-		bsdtar_errc(bsdtar, 1, 0,
+		lafe_errc(1, 0,
 		    "Option %s is not permitted in mode -%c",
 		    opt, bsdtar->mode);
 }
 
 
 void
-usage(struct bsdtar *bsdtar)
+usage(void)
 {
 	const char	*p;
 
-	p = bsdtar->progname;
+	p = lafe_progname;
 
 	fprintf(stderr, "Usage:\n");
 	fprintf(stderr, "  List:    %s -tf <archive-filename>\n", p);
@@ -645,7 +737,7 @@ version(void)
 {
 	printf("bsdtar %s - %s\n",
 	    BSDTAR_VERSION_STRING,
-	    archive_version());
+	    archive_version_string());
 	exit(0);
 }
 
@@ -685,12 +777,12 @@ static const char *long_help_msg =
  *          echo bsdtar; else echo not bsdtar; fi
  */
 static void
-long_help(struct bsdtar *bsdtar)
+long_help(void)
 {
 	const char	*prog;
 	const char	*p;
 
-	prog = bsdtar->progname;
+	prog = lafe_progname;
 
 	fflush(stderr);
 
